{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvThread.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvThread;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  JvTypes;

type
  TJvThread = class(TComponent)
  private
    FThreadCount : Integer;
    FExclusif : boolean;
    FRunOnCreate : boolean;
    FOnbegin : TNotifyEvent;
    FOnExecute : TNotifyEventParams;
    FOnFinish : TNotifyEvent;
    FOnFinishAll : TNotifyEvent;
    FFreeOnTerminate : boolean;
    procedure DoCreate;
    procedure DoTerminate(Sender : TObject);
  protected
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
  published
    function Execute(p : pointer):Thandle;
    function OneThreadIsRunning : boolean;
    function GetPriority(Thread:Thandle):TThreadPriority;
    procedure SetPriority(Thread:THandle;Priority:TThreadPriority);
    procedure QuitThread(Thread:Thandle);
    procedure Suspend(Thread:Thandle);
    procedure Resume(Thread:Thandle);
    property Exclusif : boolean read FExclusif write FExclusif;
    property RunOnCreate : boolean read FRunOnCreate write FRunOnCreate;
    property FreeOnTerminate : boolean read FFreeOnTerminate write FFreeOnTerminate;
    property Onbegin : TNotifyEvent read FOnbegin write FOnBegin;
    property OnExecute : TNotifyEventParams read FOnExecute write FOnExecute;
    property OnFinish : TNotifyEvent read FOnFinish write FOnFinish;
    property OnFinishAll : TNotifyEvent read FOnFinishAll write FOnFinishAll;
  end;

  TJvHideThread = class(TThread)
  private
    FExecuteEvent : TNotifyEventParams;
    FParams : pointer;
  public
    constructor Create(event : TNotifyEventParams; params : pointer); virtual;
    procedure Execute; override;
  end;

  procedure Synchronize(Method: TNotifyEvent);
  procedure SynchronizeParams(Method: TNotifyEventParams; p : pointer);
  
implementation

var
   mtx : THandle;

{*****************************************************}
procedure Synchronize(Method : TNotifyEvent);
begin
   WaitForSingleObject(mtx, INFINITE);
   Method(nil);
   ReleaseMutex(mtx);
end;
{*****************************************************}
procedure SynchronizeParams(Method : TNotifyEventParams; p : pointer);
begin
   WaitForSingleObject(mtx, INFINITE);
   Method(nil, p);
   ReleaseMutex(mtx);
end;
{*****************************************************}
constructor TJvThread.Create(AOwner : TComponent);
begin
   inherited Create(AOwner);
   FThreadCount:=0;
   FRunOnCreate:=true;
   FExclusif:=true;
   FreeOnTerminate:=true;
end;
{*****************************************************}
destructor TJvThread.Destroy;
begin
   inherited Destroy;
end;
{*****************************************************}
function TJvThread.Execute(p : pointer):Thandle;
var
  HideThread : TJvHideThread;
begin
   result:=0;
   if Assigned(FOnExecute) then
   begin
      if Exclusif then
        if OneThreadIsRunning then exit;
      inc(FThreadCount);
      HideThread:=TJvHideThread.Create(FOnExecute, p);
      HideThread.FreeOnTerminate:=FFreeOnTerminate;
      HideThread.OnTerminate:=DoTerminate;
      DoCreate;
      if FRunOnCreate then
         HideThread.Resume;
      result:=HideThread.ThreadID;
   end;
end;
{*****************************************************}
function TJvThread.GetPriority(Thread:Thandle):TThreadPriority;
begin
   result:=tpIdle;
   if Thread<>0 then result:=TThreadPriority(GetThreadPriority(thread));
end;
{*****************************************************}
procedure TJvThread.SetPriority(Thread:THandle;Priority:TThreadPriority);
begin
   SetThreadPriority(Thread,integer(priority));
end;
{*****************************************************}
procedure TJvThread.QuitThread(Thread:Thandle);
begin
   TerminateThread(Thread,0);
end;
{*****************************************************}
procedure TJvThread.Suspend(Thread:Thandle);
begin
   SuspendThread(Thread);
end;
{*****************************************************}
procedure TJvThread.Resume(Thread:Thandle);
begin
   ResumeThread(thread);
end;
{*****************************************************}
procedure TJvThread.DoCreate;
begin
   if Assigned(FOnBegin) then
     FOnBegin(nil);
end;
{*****************************************************}
procedure TJvThread.DoTerminate;
begin
   dec(FThreadCount);
   if Assigned(FOnFinish) then
     FOnFinish(nil);
   if FThreadCount=0 then
     if Assigned(FOnFinishAll) then
       FOnFinishAll(nil);
end;
{*****************************************************}
function TJvThread.OneThreadIsRunning : boolean;
begin
   result:=FThreadCount>0;
end;
{*****************************************************}




{*****************************************************}
constructor TJvHideThread.Create(event : TNotifyEventParams; params : pointer);
begin
   inherited Create(true);
   FExecuteEvent:=event;
   FParams:=params;
end;
{*****************************************************}
procedure TJvHideThread.Execute;
begin
   FExecuteEvent(nil, FParams);
end;
{*****************************************************}
   
initialization
  mtx:=CreateMutex(nil, false, 'VCLJvThreadMutex');
finalization
  CloseHandle(mtx);
end.
